#!/usr/bin/python

from __future__ import print_function
import sys
import os
import shutil
import pdb

LSB_RELEASE_FILE='/etc/lsb-release'
OS_RELEASE_FILE='/etc/os-release'
ISSUE_FILE='/etc/issue'
ISSUENET_FILE='/etc/issue.net'

# for test
#LSB_RELEASE_FILE='/tmp/lsb-release'
#OS_RELEASE_FILE='/tmp/os-release'
#ISSUE_FILE='/tmp/issue'
#ISSUENET_FILE='/tmp/issue.net'


distinfo = {}

def get_os_release():
    cdos_info='/etc/cdos/info'
    if os.path.exists(cdos_info):
        try:
            with open(cdos_info) as info_file:
                for line in info_file:
                    line = line.strip()
                    if not line:
                        continue
                    if line.startswith('#'):
                        continue
                    # Skip invalid lines
                    if not '=' in line:
                        continue
                    var, arg = line.split('=', 1)
                    if var.startswith('NAME'):
                        if arg.startswith('"') and arg.endswith('"'):
                            arg = arg[1:-1]
                        if arg: # Ignore empty arguments
                            distinfo[var] = arg.strip()
                    if var.startswith('VERSION'):
                        if arg.startswith('"') and arg.endswith('"'):
                            arg = arg[1:-1]
                        if arg: # Ignore empty arguments
                            distinfo[var] = arg.strip()
                    if var.startswith('VERSION_ID'):
                        if arg.startswith('"') and arg.endswith('"'):
                            arg = arg[1:-1]
                        if arg: # Ignore empty arguments
                            distinfo[var] = arg.strip()
                    if var.startswith('ARCH'):
                        if arg.startswith('"') and arg.endswith('"'):
                            arg = arg[1:-1]
                        if arg: # Ignore empty arguments
                            distinfo[var] = arg.strip()
                    if var.startswith('FEATURE_NAME'):
                        if arg.startswith('"') and arg.endswith('"'):
                            arg = arg[1:-1]
                        if arg: # Ignore empty arguments
                            distinfo[var] = arg.strip()
                    if var.startswith('ID'):
                        if arg.startswith('"') and arg.endswith('"'):
                            arg = arg[1:-1]
                        if arg: # Ignore empty arguments
                            distinfo[var] = arg.strip()
                    if var.startswith('ID_LIKE'):
                        if arg.startswith('"') and arg.endswith('"'):
                            arg = arg[1:-1]
                        if arg: # Ignore empty arguments
                            distinfo[var] = arg.strip()
                    if var.startswith('CODENAME'):
                        if arg.startswith('"') and arg.endswith('"'):
                            arg = arg[1:-1]
                        if arg: # Ignore empty arguments
                            distinfo[var] = arg.strip()
                    if var.startswith('DESKTOP'):
                        if arg.startswith('"') and arg.endswith('"'):
                            arg = arg[1:-1]
                        if arg:
                            distinfo[var] = arg.strip()
                    if var.startswith('DESCRIPTION'):
                        if arg.startswith('"') and arg.endswith('"'):
                            arg = arg[1:-1]
                        if arg: # Ignore empty arguments
                            distinfo[var] = arg.strip()
                    if var.startswith('KERNEL'):
                        if arg.startswith('"') and arg.endswith('"'):
                            arg = arg[1:-1]
                        if arg: # Ignore empty arguments
                            distinfo[var] = arg.strip()
                        else:
                            f = open('/proc/sys/kernel/osrelease')
                            version = f.readline().split('\n')[0]
                            distinfo[var] = version.strip()
                    if var.startswith('HOME_URL'):
                        if arg.startswith('"') and arg.endswith('"'):
                            arg = arg[1:-1]
                        if arg: # Ignore empty arguments
                            distinfo[var] = arg.strip()
                    if var.startswith('SUPPORT_URL'):
                        if arg.startswith('"') and arg.endswith('"'):
                            arg = arg[1:-1]
                        if arg: # Ignore empty arguments
                            distinfo[var] = arg.strip()
                    if var.startswith('BUG_REPORT_URL'):
                        if arg.startswith('"') and arg.endswith('"'):
                            arg = arg[1:-1]
                        if arg: # Ignore empty arguments
                            distinfo[var] = arg.strip()
                    if var.startswith('UPSTREAM_ID'):
                        if arg.startswith('"') and arg.endswith('"'):
                            arg = arg[1:-1]
                        if arg: # Ignore empty arguments
                            distinfo[var] = arg.strip()
                    if var.startswith('UPSTREAM_RELEASE'):
                        if arg.startswith('"') and arg.endswith('"'):
                            arg = arg[1:-1]
                        if arg: # Ignore empty arguments
                            distinfo[var] = arg.strip()
                    if var.startswith('UPSTREAM_CODENAME'):
                        if arg.startswith('"') and arg.endswith('"'):
                            arg = arg[1:-1]
                        if arg: # Ignore empty arguments
                            distinfo[var] = arg.strip()
                    if var.startswith('UPSTREAM_DESCRIPTION'):
                        if arg.startswith('"') and arg.endswith('"'):
                            arg = arg[1:-1]
                        if arg: # Ignore empty arguments
                            distinfo[var] = arg.strip()
                    if var.startswith('UPSTREAM_KERNEL'):
                        if arg.startswith('"') and arg.endswith('"'):
                            arg = arg[1:-1]
                        if arg: # Ignore empty arguments
                            distinfo[var] = arg.strip()
                    if var.startswith('GRUB_TITLE'):
                        if arg.startswith('"') and arg.endswith('"'):
                            arg = arg[1:-1]
                        if arg: # Ignore empty arguments
                            distinfo[var] = arg.strip()
        except IOError as msg:
            print('Unable to open ' + etc_cdos_release + ':', str(msg), file=sys.stderr)
    return distinfo

def update_lsb_release_file():
    """ Update /etc/lsb_release file for This OS"""
    name = distinfo.get('NAME', 'cdos')
    version_id = distinfo.get('VERSION_ID', 'n/a')
    codename = distinfo.get('CODENAME', 'cdos')
    desc = distinfo.get('DESCRIPTION', 'n/a')

    relinfo = """DISTRIB_ID=%s
DISTRIB_RELEASE=%s
DISTRIB_CODENAME=%s
DISTRIB_DESCRIPTION=\"%s\"
""" % (name, version_id, codename, desc)
    

    #if os.path.exists('/etc/lsb-release'):

    try:
        lsb_release_file = open(LSB_RELEASE_FILE, 'wb')
        try:
            lsb_release_file.write(relinfo)
        finally:
            lsb_release_file.close()
    except IOError as msg:
        print("Failed to open file %s :" % lsb_release_file, str(msg), file=sys.stderr) 

def update_os_release_file():
    """ Update /etc/os-release file for CDOS"""
    name = distinfo.get('NAME', 'CDOS')
    name_id = distinfo.get('ID', 'cdos')
    version_id = distinfo.get('VERSION_ID', 'n/a')
    arch = distinfo.get('ARCH', 'x86_64')
    fn = distinfo.get('FEATURE_NAME', 'generic')
    version = "%s %s %s" % (version_id, arch, fn)
    codename = distinfo.get('CODENAME', 'cdos')
    desktop = distinfo.get('DESKTOP', 'n/a')
    pretty_name = "%s %s %s %s" % (name, version_id, arch, fn)
    id_like = distinfo.get('ID_LIKE', 'n/a')
    
    relinfo = """NAME=%s
VERSION=\"%s\"
CODENAME=%s
ID=%s
ID_LIKE=%s
VERSION_ID=%s
PRETTY_NAME=\"%s\"
""" % (name, version, codename, name_id, id_like, version_id, pretty_name)


    try:
        os_release_file = open(OS_RELEASE_FILE, 'wb')
        try:
            os_release_file.write(relinfo)
        finally:
            os_release_file.close()
    except IOError as msg:
        print("Failed to open file %s :" % os_release_file, str(msg), file=sys.stderr) 

def update_issue_files():
    """ Update /etc/issue and /etc/issue.net files"""
    desc = distinfo.get('DESCRIPTION', 'n/a')

    try:
        issue_file = open(ISSUE_FILE, 'wb')
        try:
            issue_file.write("%s \\n \l" % desc)
        finally:
            issue_file.close()
    except IOError as msg:
        print("Failed to open file %s:" % issue_file, str(msg), file=sys.stderr)


    try:
        issuenet_file = open(ISSUENET_FILE, 'wb')
        try:
            issuenet_file.write("%s" % desc)
        finally:
            issuenet_file.close()
    except IOError as msg:
        print("Failed to open file %s: " % issuenet_file, str(msg), file=sys.stderr)


def main():
    get_os_release()
    #pdb.set_trace()
    update_lsb_release_file()
    update_os_release_file()
    update_issue_files()

    print('update base-files end.')

if __name__ == '__main__':
    main()

